--- asterisk-1.4.18.1/channels/chan_sip.c~	2008-03-19 23:36:52.000000000 -0600
+++ asterisk-1.4.18.1/channels/chan_sip.c	2008-04-30 18:07:54.000000000 -0600
@@ -11290,13 +11290,27 @@
 		struct sip_pvt *p;
 		struct sip_request req;
 		struct ast_variable *var;
+		char *content = NULL;
+		struct ast_channel *chan = NULL;
 
 		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY))) {
 			ast_log(LOG_WARNING, "Unable to build sip pvt data for notify (memory/socket error)\n");
 			return RESULT_FAILURE;
 		}
 
-		if (create_addr(p, argv[i])) {
+		/* See if they specified a channel name. */
+		if ((chan = ast_get_channel_by_name_locked(argv[i]))) {
+			if (chan->tech == &sip_tech) {
+				/* Take the address from the existing pvt. */
+				struct sip_pvt *chan_p = chan->tech_pvt;
+				p->recv = p->sa = chan_p->sa;
+			} else {
+				/* Not a SIP peer, so ignore. */
+				ast_channel_unlock(chan);
+				chan = NULL;
+			}
+		}
+		if (!chan && create_addr(p, argv[i])) {
 			/* Maybe they're not registered, etc. */
 			sip_destroy(p);
 			ast_cli(fd, "Could not create address for '%s'\n", argv[i]);
@@ -11305,8 +11319,26 @@
 
 		initreqprep(&req, p, SIP_NOTIFY);
 
-		for (var = varlist; var; var = var->next)
-			add_header(&req, var->name, ast_unescape_semicolon(var->value));
+		for (var = varlist; var; var = var->next) {
+			if (!strcasecmp(var->name, "Content")) {
+				/* They're requesting a specific content. */
+				content = var->value;
+			} else {
+				add_header(&req, var->name, ast_unescape_semicolon(var->value));
+			}
+		}
+
+		if (content) {
+			/* Expand channel variables in the content. */
+			char buf[4096];
+			if (chan) {
+				memset(buf, 0, sizeof(buf));
+				pbx_substitute_variables_helper(chan, content, buf, sizeof(buf) - 1);
+				content = buf;
+			}
+			add_header_contentLength(&req, strlen(content));
+			add_line(&req, content);
+		}
 
 		/* Recalculate our side, and recalculate Call ID */
 		if (ast_sip_ouraddrfor(&p->sa.sin_addr, &p->ourip))
@@ -11316,6 +11348,11 @@
 		ast_cli(fd, "Sending NOTIFY of type '%s' to '%s'\n", argv[2], argv[i]);
 		transmit_sip_request(p, &req);
 		sip_scheddestroy(p, DEFAULT_TRANS_TIMEOUT);
+
+		if (chan) {
+			ast_channel_unlock(chan);
+			chan = NULL;
+		}
 	}
 
 	return RESULT_SUCCESS;
@@ -11558,8 +11595,10 @@
 
 static char notify_usage[] =
 "Usage: sip notify <type> <peer> [<peer>...]\n"
-"       Send a NOTIFY message to a SIP peer or peers\n"
-"       Message types are defined in sip_notify.conf\n";
+"       Send a NOTIFY message to a SIP peer or peers.\n"
+"       Message types are defined in sip_notify.conf\n"
+"       Channel names can be used in the place of peer names\n"
+"       if variable expansion is desired on the Content field.\n";
 
 static char show_users_usage[] = 
 "Usage: sip show users [like <pattern>]\n"
